#ifndef AMDIS_CFE_INTEGRATION_H
#define AMDIS_CFE_INTEGRATION_H

#include "ElementFunction.h"
#include "Quadrature.h"
#include "ElementLevelSet.h"

namespace AMDiS {

  class CFE_Integration
  {
  public:
    /**
     * Calculates integral of function f on domain where level set function
     * is negative.
     */
    static double integrate_onNegLs(ElementFunction<double> *f, 
				    ElementLevelSet *elLS,
				    int deg = 1, 
				    Quadrature *q = NULL);

    /**
     * Calculates surface integral of function f on the zero level set.
     *
     * Note: Quadrature q is a quadrature formula for dimension dim-1.
     */
    static double integrate_onZeroLs(ElementFunction<double> *f, 
				     ElementLevelSet *elLS,
				     int deg = 1, 
				     Quadrature *q = NULL);
  protected:
    /**
     * Calculates determinant for surface given through surfVert.
     */
    static double calcSurfaceDet(ElInfo *loc_elInfo,
				 VectorOfFixVecs<DimVec<double> > &surfVert);
  };

}

#endif  // AMDIS_CFE_INTEGRATION_H
