#include "AdaptStationary.h"
#include "Parameters.h"
#include "Estimator.h"
#include "TecPlotWriter.h"
#include "ProblemIterationInterface.h"
#include <math.h>

namespace AMDiS {

  AdaptStationary::AdaptStationary(std::string name,
				   ProblemIterationInterface *prob,
				   AdaptInfo *info) 
    : AdaptBase(name, prob, info)
  {
    initialize();
  }

  int AdaptStationary::adapt()
  {
    FUNCNAME("AdaptStationary::adapt()");

    // initial iteration
    if (adaptInfo->getSpaceIteration() == -1) {
      problemIteration_->beginIteration(adaptInfo);
      problemIteration_->oneIteration(adaptInfo, NO_ADAPTION);
      problemIteration_->endIteration(adaptInfo);
      adaptInfo->incSpaceIteration();
    }

    // adaption loop
    while (!adaptInfo->spaceToleranceReached() && 
	   (adaptInfo->getSpaceIteration() < adaptInfo->getMaxSpaceIteration() || 
	    adaptInfo->getMaxSpaceIteration() < 0) ) {

      problemIteration_->beginIteration(adaptInfo);
      Flag adapted = problemIteration_->oneIteration(adaptInfo, FULL_ITERATION);
      problemIteration_->endIteration(adaptInfo);
      
      if (!adapted) 
	break;
      
      adaptInfo->incSpaceIteration();
    }

    return 0;
  }

  void AdaptStationary::initialize()
  {
    FUNCNAME("AdaptStationary::initialize()");
  
    GET_PARAMETER(0, name + "->info", "%d", &info_);
  }

}
