// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file GlobalDOFNumbering.h */

#ifndef AMDIS_GLOBALDOFNUMBERING_H
#define AMDIS_GLOBALDOFNUMBERING_H

#include <map>
#include <vector>
#include "Global.h"
#include "AMDiS_fwd.h"

namespace AMDiS {

  /** \ingroup Parallel
   * \brief 
   * Creates a global DOF numbering.
   */
  class GlobalDOFNumbering
  {
  public:
    /// Constructor
    GlobalDOFNumbering(std::vector<MeshStructure*> &meshCodes,
		       std::vector< std::vector<DegreeOfFreedom> > &dofCodes,
		       int dofsPerElement);

    /// Returns the local index of rank corresponding to global index.
    int getLocalIndex(int rank, int globalIndex);

    /// Returns the global index corresponing to local index of rank.
    int getGlobalIndex(int rank, int localIndex);

  protected:
    void handleRecursive(std::vector<MeshStructure*> &meshCodes,
			 std::vector< std::vector<DegreeOfFreedom>::iterator> &it);

  protected:
    std::vector< std::map<DegreeOfFreedom, DegreeOfFreedom> > localToGlobal_;

    std::vector< std::map<DegreeOfFreedom, DegreeOfFreedom> > globalToLocal_;

    int dofsPerElement_;

    DegreeOfFreedom globalIndexCounter_;
  };

}

#endif
