// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file GlobalElementNumbering.h */

#ifndef AMDIS_GLOBALELEMENTNUMBERING_H
#define AMDIS_GLOBALELEMENTNUMBERING_H

#include <map>
#include "AMDiS_fwd.h"

namespace AMDiS {

  /** \ingroup Parallel
   * \brief 
   * Creates a global element numbering.
   */
  class GlobalElementNumbering
  {
  public:
    /// Constructor
    GlobalElementNumbering(MeshStructure *compositeStructure,
			   Mesh *localMesh);

    /// Returns the local index corresponding to local index.
    int getLocalIndex(int globalIndex);

    /// Returns the global index corresponing to local index.
    int getGlobalIndex(int localIndex);

  protected:
    /// Maps each local index to its global index.
    std::map<int, int> localToGlobal_;

    /// Stores local index for each global index.
    std::map<int, int> globalToLocal_;
  };

}

#endif
