// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file MeshStructure.h */

#ifndef AMDIS_MESHSTRUCTURE_ED_H
#define AMDIS_MESHSTRUCTURE_ED_H

#include <vector>
#include "MeshStructure.h"
#include "ElementData.h"
#include "Element.h"

namespace AMDiS {

  const int MESH_STRUCTURE = 10;

  class MeshStructure_ED : public ElementData
  {
  public:
    MeshStructure_ED(ElementData *decorated = NULL) 
      : ElementData(decorated),
	structure_(NULL)
    {}

    virtual ~MeshStructure_ED() 
      {
      delete structure_;
    }

    virtual bool refineElementData(Element* parent, 
				   Element* child1,
				   Element* child2,
				   int elType)
    {
      ElementData::refineElementData(parent, child1, child2, elType);

      structure_->reset();

      TEST_EXIT(structure_)("no structure\n");

      TEST_EXIT(!structure_->isLeafElement())
	("don't store leaf structures\n");

      structure_->nextElement();

      if (!structure_->isLeafElement()) {
	MeshStructure *structure1 = new MeshStructure();
	structure_->skipBranch(structure1);
	structure1->commit();
	MeshStructure_ED *elData1 = new MeshStructure_ED(child1->getElementData());
	elData1->setStructure(structure1);
	child1->setElementData(elData1);
      } else {
	structure_->nextElement();
      }

      if (!structure_->isLeafElement()) {
	MeshStructure *structure2 = new MeshStructure();
	structure_->skipBranch(structure2);
	structure2->commit();
	MeshStructure_ED *elData2 = new MeshStructure_ED(child2->getElementData());
	elData2->setStructure(structure2);
	child2->setElementData(elData2);
      }

      return true;
    };

    virtual const int getTypeID() const 
    {
      return MESH_STRUCTURE;
    }

    virtual bool isOfType(int typeID) const 
    {
      if (typeID == MESH_STRUCTURE) 
	return true;
      return false;
    }

    inline void setStructure(MeshStructure *structure) 
    {
      structure_ = structure;
    }

    inline MeshStructure *getStructure() 
    {
      return structure_;
    }

  protected:
    MeshStructure *structure_;
  };

}

#endif
