// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file MpCCIAdapter.h */

#ifndef AMDIS_MPCCIADAPTER_H
#define AMDIS_MPCCIADAPTER_H

#include <map>
#include "AMDiS_fwd.h"
#include "cci.h"
#include "cci_const.h"

namespace AMDiS {

  class MpCCIAdapter
  {
  public:
    MpCCIAdapter(int                 localComm,
		 int                 meshID,
		 int                 partitionID,
		 bool                useMPI,
		 FiniteElemSpace    *feSpace,
		 int                 elementRegion,
		 int                 surfaceRegion,
		 int                 numQuantities,
		 int                *quantityIDs,
		 int                *quantityDims,
		 DOFVector<double> **dofVectors,
		 int                 numSyncPoints,
		 int                *syncPointIDs,
		 const char         *idString = NULL,
		 bool (*elementFct)(ElInfo *elInfo) = NULL,
		 bool (*surfaceFct)(ElInfo *elInfo, int side) = NULL);

    ~MpCCIAdapter();

    void remesh();

    void putNodes(int quantityID);

    void getNodes(int quantityID);

    void send(int numQuantities, int *quantityIDs);

    void isend(int numQuantities, int *quantityIDs, CCI_Request *request);

    void wait(CCI_Request *request);

    void recv(int numQuantities, int *quantityIDs);

    void reachSyncPoint(int syncPointID);

    void closeCouplingStep();

    void checkConvergence(int  myConvergence,
			  int *globalConvergence);

    void setIndexMappings(std::map<DegreeOfFreedom, DegreeOfFreedom> *newNodeIndex,
			  std::map<DegreeOfFreedom, DegreeOfFreedom> *oldNodeIndex,
			  std::map<int, int> *newElementIndex,
			  std::map<int, int> *oldElementIndex);


    int getNumElements();

  protected:
    void transferMeshToMpCCI();

  protected:
    int localCodeID_;
    int remoteCodeID_;

    int localComm_;
    int remoteComm_;

    FiniteElemSpace *feSpace_;

    int applicationID_;
    int meshID_;
    int partitionID_;
  
    SMIAdapter *smiAdapter_;

    bool useMPI_;

    int mpiRank_;
    int mpiSize_;
  };

}

#endif
