#include "ProblemInterpolScal.h"
#include "Mesh.h"
#include "DOFVector.h"
#include "Parameters.h"
#include "AdaptInfo.h"

namespace AMDiS {

  ProblemInterpolScal::ProblemInterpolScal(const char *name,
					   ProblemScal *spaceProblem,
					   AbstractFunction<double, WorldVector<double> > *fct,
					   AbstractFunction<WorldVector<double>, WorldVector<double> > *grdFct)
    : ProblemScal(name),
      interpolFct_(fct),
      grdInterpolFct_(grdFct)
  {
    Flag adoptFlag = 
      INIT_SYSTEM | 
      INIT_MESH |
      INIT_FE_SPACE;

    Flag initFlag = INIT_ALL & ~adoptFlag & ~INIT_SOLVER & ~INIT_ESTIMATOR;
  
    initialize(initFlag, spaceProblem, adoptFlag);
  }

  void ProblemInterpolScal::solve(AdaptInfo *adaptInfo) {
    mesh->dofCompress();
    solution->interpol(interpolFct_);
  }

  void ProblemInterpolScal::estimate(AdaptInfo *adaptInfo, double) {
    FUNCNAME("ProblemIterpolScal::estimate()");
    double errMax = 0.0;
    double errSum = 0.0;
    int errorNorm = 0;

    int relErr = 0;
    GET_PARAMETER(0, name + "->rel error", "%d", &relErr);

    if (grdInterpolFct_) 
      errorNorm = 1;
    else
      if (interpolFct_) 
	errorNorm = 2;

    switch (errorNorm) {
    case 1:
      errSum = Error<double>::H1Err(*grdInterpolFct_, *solution, relErr, &errMax, true);
      break;
    case 2:
      errSum = Error<double>::L2Err(*interpolFct_, *solution, relErr, &errMax, true);
      break;
    default: 
      ERROR_EXIT("invalid error norm\n");
    }

    MSG("estimate: %e\n", errSum);

    adaptInfo->setEstSum(errSum, 0);
    adaptInfo->setEstMax(errMax, 0);
  }

}
