// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ProblemInterpolVec.h */

#ifndef AMDIS_PROBLEMINTERPOLVEC_H
#define AMDIS_PROBLEMINTERPOLVEC_H

#include "ProblemVec.h"

namespace AMDiS {

  // ============================================================================
  // ===== class ProblemInterpolVec =============================================
  // ============================================================================

  /** \brief
   * Interpolates a given function adaptive on spaceProblems mesh.
   */
  class ProblemInterpolVec : public ProblemVec
  {
  public:
    /** \brief
     * Constructor. fct will be interpolated on the mesh of spaceProblem.
     * grdFct is used, if H1 error should be used for estimation. It points
     * to the gradient of fct.
     */
    ProblemInterpolVec(const char *name,
		       ProblemVec *spaceProblem,
		       std::vector<AbstractFunction<double, WorldVector<double> >*> *fct,
		       std::vector<AbstractFunction<WorldVector<double>, WorldVector<double> >*> *grdFct);

    /** \brief
     * no system assemblage.
     */
    virtual void buildbeforeRefine(AdaptInfo *adaptInfo, Flag) {}

    /** \brief
     * no system assemblage.
     */
    virtual void buildbeforeCoarsen(AdaptInfo *adaptInfo, Flag) {}

    /** \brief
     * no system assemblage.
     */
    virtual void buildAfterCoarsen(AdaptInfo *adaptInfo, Flag) {}

    /** \brief
     * no equation system ins solved. Instead fct is interpolated to uh.
     */
    virtual void solve(AdaptInfo *adaptInfo);

    /** \brief
     * True H1 or L2 error is calculated.
     */
    virtual void estimate(AdaptInfo *adaptInfo, double);

  protected:
    /** \brief
     * function to interpolate.
     */
    std::vector<AbstractFunction<double, WorldVector<double> >*> *interpolFct_;

    /** \brief
     * gradient of \ref interpolFct_. Used for H1 error in estimate().
     */
    std::vector<AbstractFunction<WorldVector<double>, WorldVector<double> >*> *grdInterpolFct_;
  };

}

#endif
