#include "ProblemNonLin.h"
#include "NonLinSolver.h"
#include "CreatorMap.h"
#include "NonLinUpdater.h"
#include "Traverse.h"
#include "AdaptInfo.h"

#if 0
namespace AMDiS {

  void ProblemNonLinScal::initialize(Flag initFlag,
				     ProblemScal *adoptProblem /*= NULL*/,
				     Flag adoptFlag /*= INIT_NOTHING*/) 
  {

    ProblemScal::initialize(initFlag, adoptProblem, adoptFlag);

    // === create Updater ===
    if (updater_) { 
      WARNING("updater already created\n");
    } else {
      if (initFlag.isSet(INIT_UPDATER) || 
	  ((!adoptFlag.isSet(INIT_UPDATER))&&
	   (initFlag.isSet(INIT_NONLIN_SOLVER)))) {
	createUpdater();
      } 
      
      if (adoptProblem && 
	  (adoptFlag.isSet(INIT_UPDATER) ||  
	   ((!initFlag.isSet(INIT_UPDATER))&&
	    (adoptFlag.isSet(INIT_NONLIN_SOLVER))))) {
	TEST_EXIT(updater_ == NULL)("updater already created\n");
	updater_ = dynamic_cast<ProblemNonLinScal*>(adoptProblem)->getUpdater();
      }
    }

    if(updater_==NULL) WARNING("no updater created\n");

    // === create nonlinear solver ===
    if (nonLinSolver_) { 
      WARNING("nonlinear solver already created\n");
    } else {
      if (initFlag.isSet(INIT_NONLIN_SOLVER)) {
	createNonLinSolver();
      } 

      if (adoptProblem && adoptFlag.isSet(INIT_NONLIN_SOLVER)) {
	TEST_EXIT(nonLinSolver_==NULL)("nonlinear solver already created\n");
	nonLinSolver_ = dynamic_cast<ProblemNonLinScal*>(adoptProblem)->getNonLinSolver();
      }
    }

    if (nonLinSolver_ == NULL) 
      WARNING("no nonlinear solver created\n");
  }

  void ProblemNonLinScal::createNonLinSolver() 
  {
    // create non-linear solver
#if 0
    std::string nonLinSolverType("no");

    GET_PARAMETER(0, name + "->nonlin solver", &nonLinSolverType);

    NonLinSolverCreator<DOFVector<double> > *nonLinSolverCreator = 
      dynamic_cast<NonLinSolverCreator<DOFVector<double> >*>(CreatorMap<NonLinSolver<DOFVector<double> > >::getCreator(nonLinSolverType));

    nonLinSolverCreator->setLinearSolver(solver);
    nonLinSolverCreator->setName(name + "->nonlin solver");
    nonLinSolverCreator->setNonLinUpdater(updater_);
    nonLinSolver_ = nonLinSolverCreator->create();
    nonLinSolver_->setVectorCreator(new  DOFVector<double>::Creator(feSpace));
#endif
  }

  void ProblemNonLinScal::solve(AdaptInfo *adaptInfo) 
  {
#if 0
    TEST_EXIT(nonLinSolver_)("no non-linear solver!\n");
    int iter = nonLinSolver_->solve(matVec, solution, rhs, leftPrecon, rightPrecon);
    adaptInfo->setSolverIterations(iter);
#endif
  }


  void ProblemNonLinScal::buildAfterCoarsen(AdaptInfo *adaptInfo, Flag) 
  {
    feSpace->getMesh()->dofCompress();
    MSG("%d DOFs for %s\n", feSpace->getAdmin()->getUsedSize(), feSpace->getName().c_str());
  
    TraverseStack stack;
    ElInfo *elInfo = stack.traverseFirst(mesh, -1, 
					 Mesh::CALL_LEAF_EL | 
					 Mesh::FILL_BOUND |
					 Mesh::FILL_COORDS |
					 Mesh::FILL_DET |
					 Mesh::FILL_GRD_LAMBDA);
    // for all elements ...
    while (elInfo) {
      if (solution->getBoundaryManager())
	solution->getBoundaryManager()->fillBoundaryConditions(elInfo, solution);
    
      elInfo = stack.traverseNext(elInfo);
    }
  }

  void ProblemNonLinVec::initialize(Flag initFlag,
				    ProblemVec *adoptProblem /*= NULL*/,
				    Flag adoptFlag /*= INIT_NOTHING*/) 
  {
    ProblemVec::initialize(initFlag, adoptProblem, adoptFlag);

    // === create Updater ===
    if (updater_) { 
      WARNING("updater already created\n");
    } else {
      if (initFlag.isSet(INIT_UPDATER) || 
	  ((!adoptFlag.isSet(INIT_UPDATER)) && initFlag.isSet(INIT_NONLIN_SOLVER))) {
	createUpdater();
      } 

      if (adoptProblem && 
	  (adoptFlag.isSet(INIT_UPDATER) ||  
	   ((!initFlag.isSet(INIT_UPDATER))&& adoptFlag.isSet(INIT_NONLIN_SOLVER)))) {
	TEST_EXIT(updater_==NULL)("updater already created\n");
	updater_ = dynamic_cast<ProblemNonLinVec*>(adoptProblem)->getUpdater();
      }
    }

    if (updater_ == NULL) 
      WARNING("no updater created\n");

    // === create nonlinear solver ===
    if (nonLinSolver_) { 
      WARNING("nonlinear solver already created\n");
    } else {
      if (initFlag.isSet(INIT_NONLIN_SOLVER)) {
	createNonLinSolver();
      } 

      if (adoptProblem && adoptFlag.isSet(INIT_NONLIN_SOLVER)) {
	TEST_EXIT(nonLinSolver_ == NULL)("nonlinear solver already created\n");
	nonLinSolver_ = dynamic_cast<ProblemNonLinVec*>(adoptProblem)->getNonLinSolver();
      }
    }

    if (nonLinSolver_ == NULL)
      WARNING("no nonlinear solver created\n");
  }

  void ProblemNonLinVec::createNonLinSolver() 
  {
#if 0
    // create non-linear solver
    std::string nonLinSolverType("no");

    GET_PARAMETER(0, name + "->nonlin solver", &nonLinSolverType);

    NonLinSolverCreator<SystemVector> *nonLinSolverCreator = 
      dynamic_cast<NonLinSolverCreator<SystemVector>*>(CreatorMap<NonLinSolver<SystemVector> >::getCreator(nonLinSolverType));

    nonLinSolverCreator->setLinearSolver(solver);
    nonLinSolverCreator->setName(name + "->nonlin solver");
    nonLinSolverCreator->setNonLinUpdater(updater_);
    nonLinSolver_ = nonLinSolverCreator->create();
    nonLinSolver_->setVectorCreator(new SystemVector::Creator("temp",
							      componentSpaces, 
							      nComponents));
#endif
  }


  void ProblemNonLinVec::solve(AdaptInfo *adaptInfo) 
  {
    TEST_EXIT(nonLinSolver_)("no non-linear solver!\n");
    nonLinSolver_->solve(matVec, solution, rhs, leftPrecon, rightPrecon);
  }

  void ProblemNonLinVec::buildAfterCoarsen(AdaptInfo *adaptInfo, Flag) 
  {
    FUNCNAME("ProblemNonLinVec::buildAfterCoarsen()");
    int nMeshes = static_cast<int>(meshes.size());

    for (int i = 0; i < nMeshes; i++) {
      meshes[i]->dofCompress();
    }

    for (int i = 0; i < nComponents; i++) {
      MSG("%d DOFs for %s\n", 
	  componentSpaces[i]->getAdmin()->getUsedSize(), 
	  componentSpaces[i]->getName().c_str());
    }
    
    TraverseStack stack;
    ElInfo *elInfo;
    
    // for all elements ...
    for (int i = 0; i < nComponents; i++) {
      elInfo = stack.traverseFirst(componentMeshes[i], -1, 
				   Mesh::CALL_LEAF_EL | 
				   Mesh::FILL_BOUND |
				   Mesh::FILL_COORDS |
				   Mesh::FILL_DET |
				   Mesh::FILL_GRD_LAMBDA);
      while (elInfo) {
	if (solution->getDOFVector(i)->getBoundaryManager()) {
	  solution->getDOFVector(i)->
	    getBoundaryManager()->fillBoundaryConditions(elInfo, solution->getDOFVector(i));
	}
	elInfo = stack.traverseNext(elInfo);
      }
    }
  }

}
#endif
