// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file RefinementManager2d.h */

#ifndef AMDIS_REFINEMENT_MANAGER_2D_H
#define AMDIS_REFINEMENT_MANAGER_2D_H

namespace AMDiS {

  /** \ingroup Adaption
   * \brief
   * Implements a RefinementManager for 2-dimensional meshes.
   */
  class RefinementManager2d : public RefinementManager
  {
  public:
    /// Calls base class constructor.
    RefinementManager2d()
      : RefinementManager()
    {}

    /// Destructor 
    virtual ~RefinementManager2d() {}

  protected:
    /// Used by \ref setNewCoords
    static int newCoordsFct(ElInfo *el_info);

    /// Implements RefinementManager::setNewCoords
    void setNewCoords();

    /** \brief
     *  gets the elements around the refinement edge with vertices  node[0] and
     *  node[1] ; refines those elements at this edge are not compatible 
     *  devisible;  			                                 
     *  dir determines the direction of the first neighbour
     *  get_refine_patch returns  1  if the domain's boundary is reached while
     *  looping around the refinement edge, otherwise  0
     */
    int getRefinePatch(ElInfo **el_info, DegreeOfFreedom *edge[2], int dir,
		       RCNeighbourList* refineList, int *n_neigh);

    /// Refines all elements in the patch.
    DegreeOfFreedom refinePatch(DegreeOfFreedom *edge[2], RCNeighbourList* refineList,
				int n_neigh, bool bound);

    /// Implements RefinementManager::refineFunction.
    ElInfo* refineFunction(ElInfo* el_info);

    /// Refines one Triangle.
    void bisectTriangle(Triangle *el, DegreeOfFreedom* newDofs[3]);
  };

}

#endif // AMDIS_REFINEMENT_MANAGER_2D_H
