#include "TecPlotWriter.h"
#include "DOFVector.h"
#include "SystemVector.h"

namespace AMDiS {

  template<>
  int TecPlotWriter<DOFVector<double> >::writeValuesFct(ElInfo* elinfo)
  {
    const DOFAdmin* admin = values->getFESpace()->getAdmin(); 
    int n0 = admin->getNumberOfPreDOFs(VERTEX);
    const DegreeOfFreedom **dof = elinfo->getElement()->getDOF();
  
    int dim = elinfo->getMesh()->getDim();
    int dow = Global::getGeo(WORLD);

    // for every vertex
    for(int i=0; i < dim+1; i++) {

      // search for coords of this vertex in the coord-list of this dof 
      DOFCoords::iterator coords = find(dofCoords[dof[i][n0]].begin(),
					dofCoords[dof[i][n0]].end(),
					elinfo->getCoord(i));

      if(coords == dofCoords[dof[i][n0]].end()) { // coords not found
	// add coords to list
	VertexInfo newCoords;
	newCoords.coord = elinfo->getCoord(i);
	newCoords.vertex_index = nv++;
	dofCoords[dof[i][n0]].push_back(newCoords);

	// write coords of vertex
	if(writeCoords) {
	  for(int j=0; j < dow; j++) {
	    (*outFile) << elinfo->getCoord(i)[j] << " ";
	  }
	}

	// write value
	(*outFile) << (*values)[dof[i][n0]] << std::endl;

      }
    }
    return 0;
  }

  template<>
  int TecPlotWriter<SystemVector>::writeValuesFct(ElInfo* elinfo)
  {
    const DOFAdmin* admin = values->getDOFVector(0)->getFESpace()->getAdmin(); 
    int n0 = admin->getNumberOfPreDOFs(VERTEX);
    const DegreeOfFreedom **dof = elinfo->getElement()->getDOF();
  
    int dim = elinfo->getMesh()->getDim();
    int dow = Global::getGeo(WORLD);

    // for every vertex
    for(int i=0; i < dim+1; i++) {

      // search for coords of this vertex in the coord-list of this dof 
      DOFCoords::iterator coords = find(dofCoords[dof[i][n0]].begin(),
					dofCoords[dof[i][n0]].end(),
					elinfo->getCoord(i));

      if(coords == dofCoords[dof[i][n0]].end()) { // coords not found
	// add coords to list
	VertexInfo newCoords;
	newCoords.coord = elinfo->getCoord(i);
	newCoords.vertex_index = nv++;
	dofCoords[dof[i][n0]].push_back(newCoords);

	// write coords of vertex
	if(writeCoords) {
	  for(int j=0; j < dow; j++) {
	    (*outFile) << elinfo->getCoord(i)[j] << " ";
	  }
	}

	// write value
	for(int j=0; j < values->getNumVectors(); j++) {
	  (*outFile) << (*(values->getDOFVector(i)))[dof[i][n0]] << " ";
	}
	(*outFile) << std::endl;
      }
    }
    return 0;
  }

  template<>
  void TecPlotWriter<DOFVector<double> >::writeVarName(int dim)
  {
    switch(Global::getGeo(WORLD)) {
    case 1:
      (*outFile) << "VARIABLES = \"x\"";
      break;
    case 2:
      (*outFile) << "VARIABLES = \"x\",\"y\"";
      break;
    case 3:
      (*outFile) << "VARIABLES = \"x\",\"y\",\"z\"";
      break;
    default:
      ERROR_EXIT("invalid dim of world\n");
    }
    (*outFile) << ",\"" << values->getName() << "\"" << std::endl;
  }

  template<>
  void TecPlotWriter<SystemVector>::writeVarName(int dim)
  {
    switch(Global::getGeo(WORLD)) {
    case 1:
      (*outFile) << "VARIABLES = \"x\"";
      break;
    case 2:
      (*outFile) << "VARIABLES = \"x\",\"y\"";
      break;
    case 3:
      (*outFile) << "VARIABLES = \"x\",\"y\",\"z\"";
      break;
    default:
      ERROR_EXIT("invalid dim of world\n");
    }

    int i, number = values->getNumVectors();
    for(i = 0; i < number; i++) {
      (*outFile) << ",\"" << values->getDOFVector(i)->getName() << "\"";
    }
    (*outFile) << std::endl;
  }

}

