#include "Tetrahedron.h"
#include "DOFAdmin.h"
#include "Mesh.h"
#include "CoarseningManager.h"
#include "FixVec.h"

namespace AMDiS {

  const unsigned char Tetrahedron::nChildEdge[3][2][2] = {{{5,4},{4,5}},
							  {{5,4},{5,4}},
							  {{5,4},{5,4}}};
  const unsigned char Tetrahedron::nChildFace[3][2][2] = {{{1,2},{2,1}},
							  {{1,2},{1,2}},
							  {{1,2},{1,2}}};
  const int Tetrahedron::childVertex[3][2][4] = {{{0,2,3,4},{1,3,2,4}},
						 {{0,2,3,4},{1,2,3,4}},
						 {{0,2,3,4},{1,2,3,4}}};
  const int Tetrahedron::childEdge[3][2][6] = {{{1,2,0,5,5,4},{4,3,0,5,5,4}},
					       {{1,2,0,5,4,5},{3,4,0,5,4,5}},
					       {{1,2,0,5,4,5},{3,4,0,5,4,5}}};
  const unsigned char Tetrahedron::adjacentChild[2][2] = {{0,1}, {1,0}};

  const signed char Tetrahedron::childOrientation[3][2] = {{1,1}, 
							   {1,-1}, 
							   {1,-1}};

  const unsigned char Tetrahedron::edgeOfDOFs[4][4] =  {{255,0,1,2},
							{0,255,3,4},
							{1,3,255,5},
							{2,4,5,255}};

  const int Tetrahedron::vertexOfEdge[6][2] = {{0,1},
					       {0,2},
					       {0,3},
					       {1,2},
					       {1,3},
					       {2,3}};
  const int Tetrahedron::vertexOfFace[4][3] = {{1,2,3},
					       {0,2,3},
					       {0,1,3},
					       {0,1,2}};


  const int Tetrahedron::sideOfChild[3][2][4] = {{{-1, 3, 1, 2},  // type 0
						  {3, -1, 2, 1}},
						 {{-1, 3, 1, 2},  // type 1
						  {3, -1, 1, 2}},
						 {{-1, 3, 1, 2},  // type 2
						  {3, -1, 1, 2}}};

  const int Tetrahedron::vertexOfParent[3][2][4] = {{{0, 2, 3, -1},  // type 0
						     {1, 3, 2, -1}},
						    {{0, 2, 3, -1},  // type 1
						     {1, 2, 3, -1}},
						    {{0, 2, 3, -1},  // type 2
						     {1, 2, 3, -1}}};

  const int Tetrahedron::edgeOfFace[4][3] = {{5, 4, 3},  // face 0
					     {5, 2, 1},  // face 1
					     {4, 2, 0},  // face 2
					     {3, 1, 0}}; // face 3

  bool Tetrahedron::hasSide(Element* sideElem) const
  {
    FUNCNAME("Tetrahedron::hasSide()");
    TEST_EXIT_DBG(sideElem->isTriangle())("called for sideElem-type != Triangle\n");
    ERROR_EXIT("not yet\n");
    return false;
  }

  int Tetrahedron::getVertexOfPosition(GeoIndex position,
				       int      positionIndex,
				       int      vertexIndex) const 
  {
    FUNCNAME("Triangle::getVertexOfPosition");
    switch(position) {
    case VERTEX:
      return positionIndex;
      break;
    case EDGE:
      return vertexOfEdge[positionIndex][vertexIndex];
      break;
    case FACE:
      return vertexOfFace[positionIndex][vertexIndex];
      break;
    case CENTER:
      return vertexIndex;
      break;
    default:
      ERROR_EXIT("invalid position\n");
      return 0;
    }
  }

  const FixVec<int, WORLD>&
  Tetrahedron::sortFaceIndices(int face, FixVec<int,WORLD> *vec) const
  {
    static MatrixOfFixVecs<FixVec<int,WORLD> > *sorted_3d = NULL;
    int no=0;
    FixVec<int,WORLD> *val = NULL;
    const int *vof = NULL;

    vof = vertexOfFace[face];

    if(NULL == sorted_3d) {
      sorted_3d = new MatrixOfFixVecs<FixVec<int,WORLD> >(3,4,7,NO_INIT);
 
      (*sorted_3d)[0][0][0]=(*sorted_3d)[0][0][1]=
	(*sorted_3d)[0][0][2]=(*sorted_3d)[1][0][0]=
	(*sorted_3d)[1][0][1]=(*sorted_3d)[1][0][2]=
	(*sorted_3d)[1][1][0]=(*sorted_3d)[1][2][1]=
	(*sorted_3d)[1][3][0]=(*sorted_3d)[1][4][2]=
	(*sorted_3d)[1][5][1]=(*sorted_3d)[1][6][2]=
	(*sorted_3d)[2][0][0]=(*sorted_3d)[2][0][1]=
	(*sorted_3d)[2][0][2]=(*sorted_3d)[2][1][0]=
	(*sorted_3d)[2][2][1]=(*sorted_3d)[2][3][0]=
	(*sorted_3d)[2][4][2]=(*sorted_3d)[2][5][1]=
	(*sorted_3d)[2][6][2]=(*sorted_3d)[3][0][0]=
	(*sorted_3d)[3][0][1]=(*sorted_3d)[3][0][2]=
	(*sorted_3d)[3][1][0]=(*sorted_3d)[3][2][1]=
	(*sorted_3d)[3][3][0]=(*sorted_3d)[3][4][2]=
	(*sorted_3d)[3][5][1]=(*sorted_3d)[3][6][2]=0;

      (*sorted_3d)[0][1][0]=(*sorted_3d)[0][2][1]=
	(*sorted_3d)[0][3][0]=(*sorted_3d)[0][4][2]=
	(*sorted_3d)[0][5][1]=(*sorted_3d)[0][6][2]=
	(*sorted_3d)[2][1][2]=(*sorted_3d)[2][2][0]=
	(*sorted_3d)[2][3][1]=(*sorted_3d)[2][4][1]=
	(*sorted_3d)[2][5][2]=(*sorted_3d)[2][6][0]=
	(*sorted_3d)[3][1][2]=(*sorted_3d)[3][2][0]=
	(*sorted_3d)[3][3][1]=(*sorted_3d)[3][4][1]=
	(*sorted_3d)[3][5][2]=(*sorted_3d)[3][6][0]=1;

      (*sorted_3d)[0][1][2]=(*sorted_3d)[0][2][0]=
	(*sorted_3d)[0][3][1]=(*sorted_3d)[0][4][1]=
	(*sorted_3d)[0][5][2]=(*sorted_3d)[0][6][0]=
	(*sorted_3d)[1][1][2]=(*sorted_3d)[1][2][0]=
	(*sorted_3d)[1][3][1]=(*sorted_3d)[1][4][1]=
	(*sorted_3d)[1][5][2]=(*sorted_3d)[1][6][0]=
	(*sorted_3d)[3][1][1]=(*sorted_3d)[3][2][2]=
	(*sorted_3d)[3][3][2]=(*sorted_3d)[3][4][0]=
	(*sorted_3d)[3][5][0]=(*sorted_3d)[3][6][1]=2;

      (*sorted_3d)[0][1][1]=(*sorted_3d)[0][2][2]=
	(*sorted_3d)[0][3][2]=(*sorted_3d)[0][4][0]=
	(*sorted_3d)[0][5][0]=(*sorted_3d)[0][6][1]=
	(*sorted_3d)[1][1][1]=(*sorted_3d)[1][2][2]=
	(*sorted_3d)[1][3][2]=(*sorted_3d)[1][4][0]=
	(*sorted_3d)[1][5][0]=(*sorted_3d)[1][6][1]=
	(*sorted_3d)[2][1][1]=(*sorted_3d)[2][2][2]=
	(*sorted_3d)[2][3][2]=(*sorted_3d)[2][4][0]=
	(*sorted_3d)[2][5][0]=(*sorted_3d)[2][6][1]=3;
    }

    if (dof[vof[0]][0] < dof[vof[1]][0])
      no++;
    if (dof[vof[1]][0] < dof[vof[2]][0])
      no += 2;
    if (dof[vof[2]][0] < dof[vof[0]][0])
      no += 4;

    if (!(no >= 1  &&  no <= 6)){
      ERROR_EXIT("can not sort face indices of element %d at face %d\n", 
		 getIndex(), face);
    }

    if (vec) {
      val = vec;
      *val = (*sorted_3d)[face][no];
    }
    else
      val = &((*sorted_3d)[face][no]);

    return(*(const_cast<const FixVec<int,WORLD>* >(val)));
  }

}
