#include "TraverseParallel.h"
#include "OpenMP.h"
#include "Mesh.h"

#ifdef _OPENMP

namespace AMDiS {

  TraverseParallelStack::TraverseParallelStack(int nThreads)
  {
    if (nThreads == 0) {
      nThreads_ = omp_get_overall_max_threads();
    } else {
      nThreads_ = nThreads;
    }

    stacks_.resize(nThreads_);
    int i = 0;
    for (std::vector<TraverseStack*>::iterator it = stacks_.begin();
	 it != stacks_.end();
	 ++it) {

      (*it) = new TraverseStack();

      (*it)->setMyThreadId(i++);
      (*it)->setMaxThreads(nThreads_);
    }
  }

  TraverseParallelStack::~TraverseParallelStack()
  {
    for (std::vector<TraverseStack*>::iterator it = stacks_.begin();
	 it != stacks_.end();
	 ++it) {

      delete (*it);
    }
  }

  ElInfo* TraverseParallelStack::traverseFirst(Mesh *mesh, int level, Flag fill_flag)
  {
    TEST_EXIT(fill_flag.isSet(Mesh::CALL_LEAF_EL))("not yet implemented");
    TEST_EXIT(stacks_[omp_get_thread_num()])("something wrong with parallel stack traverse");

    return stacks_[omp_get_thread_num()]->traverseFirst(mesh, level, fill_flag);
  }
}

#endif // _OPENMP
