#include "Triangle.h"
#include "DOFAdmin.h"
#include "Mesh.h"
#include "CoarseningManager.h"
#include "FixVec.h"

namespace AMDiS {

  const int Triangle::vertexOfEdge[3][2] = {{1,2}, {2,0}, {0,1}};
  const int Triangle::sideOfChild[2][3] = {{-1,2,0}, {2,-1,1}};
  const int Triangle::vertexOfParent[2][3] = {{2,0,-1}, {1,2,-1}};

  bool Triangle::hasSide(Element* sideElem) const
  {
    FUNCNAME("Triangle::hasSide");
    TEST_EXIT_DBG(sideElem->isLine())("called for sideElem-type != Line\n");
    ERROR_EXIT("not yet\n");
    return false;
  }

  int Triangle::getVertexOfPosition(GeoIndex position,
				    int      positionIndex,
				    int      vertexIndex) const 
  {
    FUNCNAME("Triangle::getVertexOfPosition");
    switch(position) {
    case VERTEX:
      return positionIndex;
      break;
    case EDGE:
      return vertexOfEdge[positionIndex][vertexIndex];
      break;
    case CENTER:
      return vertexIndex;
      break;
    default:
      ERROR_EXIT("invalid position\n");
      return 0;
    }
  }

  const FixVec<int, WORLD>&
  Triangle::sortFaceIndices(int face, FixVec<int,WORLD> *vec) const
  {
    static MatrixOfFixVecs<FixVec<int,WORLD> > *sorted_2d = NULL;

    int no = 0;
    const int *vof = NULL;
    FixVec<int,WORLD> *val = NULL;
    vof = vertexOfEdge[face];

    if (NULL == sorted_2d) {
      sorted_2d = new MatrixOfFixVecs<FixVec<int,WORLD> >(2, 3, 2, NO_INIT);

      (*sorted_2d)[1][0][1] = (*sorted_2d)[1][1][0] =
	(*sorted_2d)[2][0][0] = (*sorted_2d)[2][1][1] = 0;
      (*sorted_2d)[0][0][0] = (*sorted_2d)[0][1][1] =
	(*sorted_2d)[2][0][1] = (*sorted_2d)[2][1][0] = 1;
      (*sorted_2d)[0][0][1] = (*sorted_2d)[0][1][0] =
	(*sorted_2d)[1][0][0] = (*sorted_2d)[1][1][1] = 2;
    }

    if (dof[vof[0]][0] < dof[vof[1]][0])
      no = 0;
    else
      no = 1;

    if (vec) {
      val = vec;
      *val = (*sorted_2d)[face][no];
    } else {
      val = &((*sorted_2d)[face][no]);
    }

    return *(const_cast<const FixVec<int,WORLD>* >(val));
  }

}
