// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file UmfPackSolver.h */

#ifndef AMDIS_UMFPACKSOLVER_H
#define AMDIS_UMFPACKSOLVER_H

#if defined HAVE_UMFPACK && defined MTL_HAS_UMFPACK

#include <boost/numeric/mtl/interface/umfpack_solve.hpp>
#include "OEMSolver.h"

namespace AMDiS {

  /**
   * \ingroup Solver
   * 
   * \brief
   * Wrapper for the external UMFPACK solver:
   *   http://www.cise.ufl.edu/research/sparse/umfpack/
   *
   * This is a direct solver for large sparse matrices.
   */

  class UmfPackSolver : public OEMSolver
  {
  public:
    /// Creator class used in the OEMSolverMap.
    class Creator : public OEMSolverCreator
    {
    public:
      virtual ~Creator() {}
      
      /** \brief
       * Returns a new UmfPackSolver object.
       */
      OEMSolver* create() 
      { 
	return new UmfPackSolver(this->name); 
      }
    };

    /// Constructor
    UmfPackSolver(::std::string name) 
      : OEMSolver(name), 
	solver(0),
	store_symbolic(0)
    {
	GET_PARAMETER(0, name + "->store symbolic", "%d", &store_symbolic);
	// GET_PARAMETER(0, name + "->multiple rhs", "%d", &multipleRhs);
    }
    
    /// Destructor
    ~UmfPackSolver() 
    { 
      if (solver) 
	delete solver;
    }

    /// Solves the system directly
    int solveSystem(const DOFMatrix::base_matrix_type&      A,
		    mtl::dense_vector<value_type>&          x,
		    const mtl::dense_vector<value_type>&    b)
    {
	if (!solver)
	    solver= new mtl::matrix::umfpack::solver<matrix_type>(A);
	else if (store_symbolic)
	    solver->update_numeric();
	else
	    solver->update();
	return (*solver)(x, b);	
    }

    
  private:
    mtl::matrix::umfpack::solver<matrix_type> *solver;

    int                                       store_symbolic;
  };
  
}

#endif // HAVE_UMFPACK

#endif // AMDIS_UMFPACKSOLVER_H
