#include "parareal/ProblemBase.h"
#include "parareal/AdaptParaReal.h"

namespace AMDiS {
  int AdaptParaReal::adapt()
  {    
    // Compute number of coarse and fine time stages.
    int coarseStages = static_cast<int>(round(adaptInfo->getEndTime() / coarseTimestep));
    int fineStages = static_cast<int>(round(coarseTimestep / fineTimestep));

    // First, sequentially calculate first approximation on coarse time grid.
    adaptInfo->setTimestep(coarseTimestep);
    pararealProb->setStoreSolutions(false, true);
    pararealProb->setStoreInitSolution(true);
    pararealProb->setInitSolutionVec(NULL);
    AdaptInstationary::adapt();

    // ParaReal iterations
    for (int k = 0; k < pararealIter; k++) {
      // Compute solutions with fine timestep on the coarse grid slides.
      for (int i = 0; i < coarseStages; i++) {
	adaptInfo->resetTimeValues(fineTimestep,
				   i * coarseTimestep,
				   (i + 1) *coarseTimestep);
	pararealProb->setStoreSolutions(true, false);
	pararealProb->setStoreInitSolution(false);
	pararealProb->setInitSolutionVec((*pararealProb->getCoarseSolutions())[i]);
	pararealProb->cleanUp();
	
	AdaptInstationary::adapt();
      }
      
      // Copy coarse grid solutions to another vector.
      std::vector<DOFVector<double> *> coarseSolutions =
	(*pararealProb->getCoarseSolutions());
      pararealProb->clearCoarseSolutions();          
      (*pararealProb->getCoarseSolutions()).push_back(coarseSolutions[0]);
      
      // Again, compute solutions on coarse grid and calculate the corrected solutions.
      for (int i = 1; i <= coarseStages; i++) {
	adaptInfo->resetTimeValues(coarseTimestep,
				    (i - 1) * coarseTimestep,
				    i * coarseTimestep);
	pararealProb->setStoreSolutions(false, true);
	pararealProb->setStoreInitSolution(false);
	pararealProb->setInitSolutionVec((*pararealProb->getCoarseSolutions())[i - 1]);
	
	AdaptInstationary::adapt();
	
	DOFVector<double>* tmp = (*pararealProb->getCoarseSolutions())[i];
	*tmp += *(*pararealProb->getFineSolutions())[(i * fineStages) - 1];
	*tmp -= *(coarseSolutions[i]);
      }
    }
      
    return 0;
  }

}
